package com.contentsquare.flutter

/**
 * Executes a function and catches any exception or throwable.
 *
 * @param function The function to execute.
 * @param throwableCallback The callback to execute if a throwable is thrown.
 * @param printTrace Whether to print the stack trace of the exception or throwable.
 *
 */
internal fun safeExecute(
        function: () -> Unit,
        throwableCallback: ((Throwable) -> Unit)? = null,
        printTrace: Boolean = true,
) {
    try {
        function()
    } catch (t: Throwable) {
        throwableCallback?.invoke(t)
        if (printTrace) t.printStackTrace()
    }
}