import ContentsquareErrorAnalysisModule

enum HttpMetricConverterError: Error {
    case missingValue(forKey: String)
    case invalidURL(value: String)
}

class DictionaryToHttpMetricConverter {
    static func convertFrom(dict: [String : Any]) throws -> HTTPMetric {
        let url = try getUrl(dict:dict)

        let statusCode = try getValue(from: dict, key: "statusCode", type: Int.self)
        let method = try getValue(from: dict, key: "method", type: String.self)
        let requestTimeInMs = try getValue(from: dict, key: "requestTimeInMs", type: Int.self)
        let responseTimeInMs = try getValue(from: dict, key: "responseTimeInMs", type: Int.self)
        let requestHeadersOptional = dict["requestHeaders"] as? [String : String]
        let responseHeadersOptional = dict["responseHeaders"] as? [String : String]

        // create the metric
        let httpMetric = HTTPMetric(url: url, httpMethod: method)

        httpMetric.setStatusCode(statusCode)
        if let requestHeaders = requestHeadersOptional {
            httpMetric.setRequestHeaders(requestHeaders)
        }
        if let responseHeaders = responseHeadersOptional {
            httpMetric.setResponseHeaders(responseHeaders)
        }

        // set the request time via private API
        let selectorSetRequestTimeInMS = NSSelectorFromString("_setRequestTimeInMS:")
        if HTTPMetric.self.responds(to: selectorSetRequestTimeInMS) {
            HTTPMetric.perform(selectorSetRequestTimeInMS, with: requestTimeInMs)
        }

        // set the response time via private API
        let selectorSetResponseTimeInMS = NSSelectorFromString("_setResponseTimeInMS:")
        if HTTPMetric.self.responds(to: selectorSetResponseTimeInMS) {
            HTTPMetric.perform(selectorSetResponseTimeInMS, with: responseTimeInMs)
        }

        return httpMetric
    }

    static private func getUrl(dict: [String : Any]) throws -> URL {
        let urlString = try getValue(from: dict, key: "url", type: String.self)
        let query = dict["query"] as? String

        if let query = query, let urlWithQuery = URL(string: "\(urlString)?\(query)") {
            return urlWithQuery
        } else if let url = URL(string: urlString) {
            return url
        } else {
            throw HttpMetricConverterError.invalidURL(value: urlString)
        }
    }

    static private func getValue<T>(from dict: [String: Any], key: String, type: T.Type) throws -> T {
        guard let value = dict[key] as? T else {
            throw HttpMetricConverterError.missingValue(forKey: key)
        }
        return value
    }
}
