/// Execute the callback function and catch any error that occurs.
/// 
/// - Parameter function: The callback that will be executed  
/// - Parameter errorCallback: A nullable callback called when an error occurs
/// - Parameter printTrace: When true, the error's stack trace will be printed
internal func safeExecute(
    function: () throws -> Void,
    errorCallback: ((Error) -> Void)? = nil,
    printTrace: Bool = true
) {
    do {
        try function()
    } catch {
        errorCallback?(error)
        if printTrace {
            print("Stack trace:\n\(Thread.callStackSymbols.joined(separator: "\n"))")
        }
    }
}