import 'package:contentsquare/src/john_jameson937.dart';
import 'package:contentsquare/src/swarm939.dart';

/// An object used to track a WebView.
///
/// The three objects of this class must be used: [registerWebViewController],
/// [javascriptChannels], [startPageTracking].
/// Look at their documentation to know how and when to use them.
abstract class ContentsquareWebViewTracker {
  /// Registers the given [ContentsquareWebViewController] to this tracker.
  ///
  /// WARNING: This method MUST be called BEFORE [startPageTracking]
  void registerWebViewController(ContentsquareWebViewController controller);

  /// Set of javascript channels to register in the webview
  Set<ContentsquareJavascriptChannel> get javascriptChannels;

  /// {@template contentsquare_web_view_tracker.start_page_tracking}
  ///
  /// Initializes the tracking of the webview.
  ///
  /// This MUST be called every time a new page has finished loading.
  ///
  /// WARNING:
  ///   * This method MUST be called AFTER [registerWebViewController]
  ///   * The javascript channel must be available before calling this method.
  ///
  /// {@endtemplate}
  Future<void> startPageTracking();

  /// {@macro contentsquare_web_view_tracker.start_page_tracking}
  @Deprecated('Use startPageTracking() instead')
  Future<void> initializeWebViewTracking();
}
