/// A class which contains the methods that are needed to communicate with the
/// webview.
class ContentsquareWebViewController {
  /// Builds a [ContentsquareWebViewController]
  ContentsquareWebViewController({
    required this.evaluateJavascript,
    required this.javascriptChannelCall,
    this.currentUrl,
    this.isChannelAvailable,
  });

  /// Calls `javascriptString` in the webview and returns the result.
  final Future<String> Function(String javaScriptString) evaluateJavascript;

  /// Returns a [String] which can be used in [evaluateJavascript] to
  /// call the channel with the given data.
  ///
  /// WARNING: Based on the webview plugin you use, the format of the string
  /// might differ. Please refer to the documentation to know the format of the
  /// String you should return.
  ///
  /// Example for the webview_flutter plugin:
  /// ```dart
  /// javascriptChannelCall: (channelName, data) => 'widget.$channelName.postMessage($data)',
  /// ```
  final String Function(String channelName, String data) javascriptChannelCall;

  /// Returns the current url of the webview.
  ///
  /// Note: This is only used for logging, and therefore is optional.
  final Future<String?> Function()? currentUrl;

  /// Returns true when the channel is available.
  ///
  /// Note: This is only used for sanity checks, and therefore is optional.
  final Future<bool> Function(String channelName)? isChannelAvailable;
}
