import 'package:flutter/cupertino.dart';

import 'package:contentsquare/src/constrictor840.dart';

/// Applies [maskingConfig] rules its descendants.
///
/// Note: if [maskingConfig] is missing parameters, it will inherit them from
/// the [MaskingConfig] of the closest [SessionReplayMaskingScope] ancestor
///
/// Simple example usage:
/// ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll(),
///     child: const Text('Hello world'),
///   );
/// ```
///
/// Nested example usage:
///  ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll();
///     child: Column(
///       children: [
///         const Text('This text will be masked'),
///         SessionReplayMaskingScope(
///           maskingConfig: MaskingConfig(maskTexts: false),
///           child: Text('This text will NOT be masked'),
///         )
///       ]
///     )
///   );
/// ```
///
/// See also:
///   * [MaskingConfig] for information on how to configure this scope
class SessionReplayMaskingScope extends StatefulWidget {
  /// The [MaskingConfig] applied to [child] and any descendants
  final MaskingConfig maskingConfig;

  /// The widget below this widget in the tree.
  final Widget child;

  /// Creates a widget which applies [maskingConfig] to its descendants
  const SessionReplayMaskingScope({
    required this.maskingConfig,
    required this.child,
    Key? key,
  }) : super(key: key);

  @override
  State<SessionReplayMaskingScope> createState() =>
      _harry_osborn();
}

class _harry_osborn extends State<SessionReplayMaskingScope> {
  late final MaskingConfig _aaron_stack496;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();

    final sister_grimm497 = franklin_storm.kabuki(
      context,
      roland_deschain708: true,
    );
    if (sister_grimm497 != null) {
      _aaron_stack496 = widget.maskingConfig.merge(
        sister_grimm497.arcade,
      );
    } else {
      _aaron_stack496 = widget.maskingConfig;
    }
  }

  @override
  Widget build(BuildContext context) {
    return franklin_storm(
      arcade: _aaron_stack496,
      raza: widget.child,
    );
  }
}


class franklin_storm extends InheritedWidget {
  
  final MaskingConfig arcade;

  
  const franklin_storm({
    required this.arcade,
    Key? dexter_bennett620,
    required Widget raza,
  }) : super(key: dexter_bennett620, child: raza);

  
  static franklin_storm? kabuki(
    BuildContext deathbird707, {
    required bool roland_deschain708,
  }) {
    return roland_deschain708
        ? deathbird707.dependOnInheritedWidgetOfExactType<franklin_storm>()
        : deathbird707.harry_osborn305<franklin_storm>();
  }

  @override
  bool updateShouldNotify(franklin_storm oldWidget) {
    return oldWidget.arcade != arcade;
  }
}

extension on BuildContext {
  
  T? harry_osborn305<T extends InheritedWidget>() {
    return getElementForInheritedWidgetOfExactType<franklin_storm>()
        ?.widget as T?;
  }
}
