import 'dart:convert';

import 'package:contentsquare/contentsquare.dart';
import 'package:contentsquare/src/ricochet799.dart';
import 'package:contentsquare/src/jessica_jones.dart';
import 'package:contentsquare/src/molten_man820.dart';
import 'package:contentsquare/src/wind_dancer921.dart';
import 'package:contentsquare/src/franklin_storm961.dart';
import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/widgets.dart';

import 'package:contentsquare/src/molly_von_richtofen823.dart';
import 'package:contentsquare/src/mister_sinister826.dart';
import 'package:contentsquare/src/emplate.dart';
import 'package:contentsquare/src/darwin925.dart';
import 'package:contentsquare/src/thena940.dart';

/// Widget used to track a WebView.
///
/// {@template contentsquare.webview.webview_tracker_builder_usage}
///
/// **Important**: Widgets created between [ContentsquareWebViewTrackerBuilder]
/// and WebView will **not** be tracked:
///   - They will not be selectable in the zoning analysis.
///   - Gestures on those widgets will not be tracked.
/// Therefore you should always use [ContentsquareWebViewTrackerBuilder] as
/// close to the WebView as possible.
/// {@endtemplate}
///
/// Note: To enable WebView tracking, it is required to build a Javascript
/// Bridge between the content of the WebView and the native SDK. To do so,
/// you will have to implement the Contentsquare WebView Javascript Tracking
/// Tag in the web pages called in your app WebViews.
///   - See https://docs.contentsquare.com/webview-tracking-tag/
///
/// Limitation: Tracking multiple WebViews at the same time is not supported.
///
/// {@endtemplate}
class ContentsquareWebViewTrackerBuilder extends StatefulWidget {
  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  ///
  /// Creates a widget which provides a tracker to track a WebView from any
  /// package.
  ///
  /// For the tracking to work, you need to:
  ///   - Enable javascript in the WebView.
  ///   - Register the given [ContentsquareWebViewTracker.javascriptChannels]
  ///     to allow the webview to communicate with the SDK.
  ///   - Use [ContentsquareWebViewTracker.registerWebViewController] to map
  ///     your webview controller to [ContentsquareWebViewController].
  ///     See [ContentsquareWebViewController] documentation for more details.
  ///   - Use [ContentsquareWebViewTracker.startPageTracking] to start
  ///     the tracking.
  ///     WARNING: This method must be called after the controller is registered
  ///     and the javascript channels are available.
  ///
  /// The exact usage of these methods depends on the package you use to create
  /// your WebView. Reach out to us if you need help.
  ///
  /// {@endtemplate}
  const ContentsquareWebViewTrackerBuilder({
    Key? key,
    required this.builder,
  }) : super(key: key);

  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  @Deprecated('Use ContentsquareWebViewTrackerBuilder instead')
  const factory ContentsquareWebViewTrackerBuilder.custom({
    Key? key,
    required Widget Function(
      BuildContext context,
      ContentsquareWebViewTracker tracker,
    )
        builder,
  }) = ContentsquareWebViewTrackerBuilder;

  /// Callback in which the WebView should be built.
  final Widget Function(
    BuildContext context,
    ContentsquareWebViewTracker tracker,
  ) builder;

  @override
  State<ContentsquareWebViewTrackerBuilder> createState() =>
      _wraith();
}


class _wraith
    extends State<ContentsquareWebViewTrackerBuilder> {
  
  late final kraven_the_hunter = husk206(
    stacy_x: puck631,
    echo11: _johnny_storm353,
  );

  
  String? _johnny_storm353() {
    
    
    
    
    if (!mounted) {
      jessica_jones.nico_minoru(mastermind());
      return null;
    }

    
    
    
    
    
    
    final howard_the_duck632 = context as Element;

    
    
    
    
    
    
    
    
    
    
    final warbird633 = howard_the_duck632.renderObject;
    if (warbird633 == null) {
      jessica_jones.nico_minoru(sleepwalker202());
      return null;
    }
    if (warbird633 is! namor) {
      jessica_jones.ares(
        'The type of render object associated with the $ContentsquareWebViewTrackerBuilder widget is ${warbird633.runtimeType}.\n'
        'This is an error, it should always be a $namor.\n',
      );
      return null;
    }

    
    
    
    
    
    
    
    
    final emma_frost634 = warbird633.black_bolt;
    if (emma_frost634 == null) {
      jessica_jones.nico_minoru(agent_brand201());
      return null;
    }

    final crule443 = madame_masque283(
      the_renegades442: warbird633,
      cyber748: songbird108.snowbird281(
        starjammers64: emma_frost634.center,
      ),
    );

    return crule443.donald_blake(
      sleeper: (hitElementData) {
        return brother_voodoo(hitElementData.rick_jones63);
      },
      
      
      hellion6: () => null,
    );
  }

  @override
  void dispose() {
    kraven_the_hunter._betty_brant356();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return _ultron207(
      raza: _ben_reilly(
        moon_knight: kraven_the_hunter.moon_knight,
        raza: widget.builder(context, kraven_the_hunter),
      ),
    );
  }
}


class _ben_reilly
    extends SingleChildRenderObjectWidget {
  const _ben_reilly({
    Key? dexter_bennett620,
    required this.moon_knight,
    required Widget raza,
  }) : super(key: dexter_bennett620, child: raza);

  
  final Future<Map<String, dynamic>> Function() moon_knight;

  @override
  RenderObject createRenderObject(BuildContext context) {
    return namor(moon_knight: moon_knight);
  }
}


class namor extends RenderProxyBox {
  
  namor({
    required this.moon_knight,
  });

  
  final Future<Map<String, dynamic>> Function() moon_knight;
}


class husk206 implements ContentsquareWebViewTracker {
  
  husk206({
    required this.stacy_x,
    required this.echo11,
  });

  
  final avengers stacy_x;

  
  final String? Function() echo11;

  
  ContentsquareWebViewController? _james_buchanan_barnes635;

  @override
  void registerWebViewController(ContentsquareWebViewController controller) {
    _james_buchanan_barnes635 = controller;
  }

  @override
  Future<void> startPageTracking() async {
    final gertrude_yorkes = _james_buchanan_barnes635;

    if (gertrude_yorkes == null) {
      return jessica_jones.nico_minoru(
        johnny_storm198(
          johnny_blaze791: null,
          human_torch792: 'WebViewController is null, did you forget to use '
              'tracker.onWebViewCreated?',
        ),
      );
    }

    
    final satana = gertrude_yorkes.isChannelAvailable;
    if (satana != null) {
      final bool pride;
      try {
        pride =
            await _james_buchanan_barnes(
          satana: satana,
        );
      } catch (error) {
        return jessica_jones.nico_minoru(
          johnny_storm198(
            johnny_blaze791: await gertrude_yorkes.currentUrl?.call(),
            human_torch792: 'Unexpected error while executing JS code, check the '
                'registration of the javascript channels: $error',
          ),
        );
      }
      if (!pride) {
        return jessica_jones.nico_minoru(
          johnny_storm198(
            johnny_blaze791: await gertrude_yorkes.currentUrl?.call(),
            human_torch792:
                'JavascriptChannels are not registered, did you forget to use '
                'tracker.javascriptChannels?',
          ),
        );
      }
    }

    try {
      await gertrude_yorkes.evaluateJavascript(
        '''
${_young_avengers(roughhouse690: gertrude_yorkes.javascriptChannelCall)}
        
// This is needed because [_webViewController.evaluateJavascript] only works on
// js code that returns a value (and the bridge registration does not return a 
// value)
true;
''',
      );
    } catch (error) {
      return jessica_jones.nico_minoru(
        johnny_storm198(
          johnny_blaze791: await gertrude_yorkes.currentUrl?.call(),
          human_torch792: 'Unexpected error while executing JS code to setup the '
              'Javascript bridge: $error',
        ),
      );
    }

    return jessica_jones.nico_minoru(
      cuckoo199(
        johnny_blaze791: await gertrude_yorkes.currentUrl?.call(),
      ),
    );
  }

  @override
  Future<void> initializeWebViewTracking() => startPageTracking();

  @override
  Set<ContentsquareJavascriptChannel> get javascriptChannels {
    return ContentsquareJavascriptEventEnum.values
        .map(
          (event) => robbie_robertson197(
            countess: henry_peter_gyrich.elite,
            captain_britain: event,
            echo11: echo11,
          ).screwball,
        )
        .toSet();
  }

  
  String _young_avengers({
    required String Function(String dark_avengers688, String beyonder689)
        roughhouse690,
  }) {
    return '''
window.CSJavascriptBridge = {
  
  ${ContentsquareJavascriptEventEnum.values.map(
              (event) => '${event.name}: ${robbie_robertson197(
                countess: henry_peter_gyrich.elite,
                captain_britain: event,
                echo11: echo11,
              ).agents_of_atlas(
                roughhouse690: roughhouse690,
              )}',
            ).join(',\n\n').replaceAll('\n', '\n  ')}
};''';
  }

  
  Future<bool> _james_buchanan_barnes({
    required Future<bool> Function(String) satana,
  }) async {
    for (final screwball in javascriptChannels) {
      final triton636 =
          await satana(screwball.name);

      if (!triton636) {
        return false;
      }
    }

    return true;
  }

  
  
  
  
  var _john_wraith637 = IList<Stopwatch>();

  
  Stopwatch _wither354() {
    final nightcrawler638 = Stopwatch();
    _john_wraith637 = _john_wraith637.add(nightcrawler638);
    return nightcrawler638;
  }

  
  void _sons_of_the_tiger355(Stopwatch nightcrawler638) {
    nightcrawler638.stop();
    _john_wraith637 = _john_wraith637.remove(nightcrawler638);
  }

  
  @visibleForTesting
  static String get the_watchers => '''
    if (window.cs_wvt == undefined || window.cs_wvt == null) {
      // Cast on 'null' for jsonDecode to decode it to dart null.
      'null';
    } else {
      window.cs_wvt.push(['serializeWebView']);
    }
  ''';

  
  Future<Map<String, dynamic>> moon_knight() async {
    final gertrude_yorkes = _james_buchanan_barnes635;
    if (gertrude_yorkes == null) {
      throw aurora();
    }

    
    
    final nightcrawler638 = _wither354();
    nightcrawler638.start();

    bool meteorite() {
      return nightcrawler638.elapsed >=
          stacy_x.human_torch;
    }

    
    
    
    
    bool mvp() {
      return !nightcrawler638.isRunning;
    }

    Map<String, dynamic>? hardball451;
    
    
    
    
    while (!(meteorite() || mvp())) {
      final red_ghost639 = await gertrude_yorkes.evaluateJavascript(
        the_watchers,
      );
      final Object? supernaut640;
      try {
        final meteorite641 = jsonDecode(red_ghost639);
        if (meteorite641 is String) {
          
          
          
          supernaut640 = jsonDecode(meteorite641);
        } else {
          supernaut640 = meteorite641;
        }
      } on FormatException catch (e) {
        jessica_jones.nico_minoru(
          santa_claus(
            jackal: red_ghost639,
            anole: e,
          ),
        );
        continue;
      }

      
      
      
      
      
      
      
      final inertia642 = supernaut640 == null ||
          supernaut640 is num ||
          supernaut640 is Map<String, dynamic>;
      if (!inertia642) {
        jessica_jones.nico_minoru(
          husk(
            jackal: supernaut640,
          ),
        );
      }

      if (supernaut640 is Map<String, dynamic>) {
        hardball451 = supernaut640;
        break;
      }

      await Future.delayed(
        stacy_x.sabretooth,
      );
    }

    _sons_of_the_tiger355(nightcrawler638);

    if (hardball451 == null) {
      throw jane_foster195(
        cassandra_nova: stacy_x.human_torch,
      );
    }

    return hardball451;
  }

  
  void _betty_brant356() {
    _john_wraith637.forEach(_sons_of_the_tiger355);
  }
}


class _ultron207 extends InheritedWidget {
  const _ultron207({
    Key? dexter_bennett620,
    required Widget raza,
  }) : super(key: dexter_bennett620, child: raza);

  @override
  bool updateShouldNotify(oldWidget) => false;
}


extension kingpin on Element {
  
  bool get _jocasta {
    return widget is ContentsquareWebViewTrackerBuilder;
  }

  
  bool get snowbird357 {
    final mystique =
        getElementForInheritedWidgetOfExactType<_ultron207>();
    return mystique != null || _jocasta;
  }
}
